(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*                  projet Estime, INRIA-Rocquencourt                  *)
(*                                                                     *)
(*  Copyright 2004-2011 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: value_type_def.ml,v 1.2 2011-02-23 17:43:29 fclement Exp $ *)

(* The private type of value types *)
type value_type =
   | Scalar_type of scalar_type
   | Vector_type of scalar_type
   | Matrix_type of scalar_type

and scalar_type =
  | Bool_type
  | String_type
  | Int_type | Native_int_type | Int32_type | Int64_type
  | Float_type | Float_bin_type
  | Couple_type of scalar_type * scalar_type
  | Triple_type of scalar_type * scalar_type * scalar_type;;

(* public construction functions *)
let bool_type = Bool_type
and string_type = String_type
and int_type = Int_type
and native_int_type = Native_int_type
and int32_type = Int32_type
and int64_type = Int64_type
and float_type = Float_type
and float_bin_type = Float_bin_type
;;

let value_type_construction_equal_error () =
  failwith "Type_value: types in tuples should be equal"

and value_type_construction_tuple_error () =
  failwith "Type_value: types in tuples should be basic"

and value_type_construction_string_error () =
  failwith "Type_value: string tuples are forbidden"

and value_type_construction_matrix_error () =
  failwith "Type_value: only float matrices are allowed"
;;

let couple_type (t1, t2) =
  if t1 <> t2 then value_type_construction_equal_error () else
  match t1 with
  | Couple_type (_, _) -> value_type_construction_tuple_error ()
  | Triple_type (_, _, _) -> value_type_construction_tuple_error ()
  | String_type -> value_type_construction_string_error ()
  | Bool_type
  | Int_type | Native_int_type | Int32_type | Int64_type
  | Float_type | Float_bin_type as t1 -> Couple_type (t1, t2)
;;

let triple_type (t1, t2, t3) =
  if t1 <> t2 || t2 <> t3 then value_type_construction_equal_error () else
  match t1 with
  | Couple_type (_, _) -> value_type_construction_tuple_error ()
  | Triple_type (_, _, _) -> value_type_construction_tuple_error ()
  | String_type -> value_type_construction_string_error ()
  | Bool_type
  | Int_type | Native_int_type | Int32_type | Int64_type
  | Float_type | Float_bin_type as t1 -> Triple_type (t1, t2, t3);;

let scalar_type sc = Scalar_type sc;;

let vector_type sc = Vector_type sc;;

let matrix_type = function
  | (Float_type | Float_bin_type) as sc -> Matrix_type sc
  | Bool_type
  | String_type
  | Int_type | Native_int_type | Int32_type | Int64_type
  | Couple_type _
  | Triple_type _ -> value_type_construction_matrix_error ()
;;
