CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                            Pio (Fortran)                            C
C                                                                     C
C                    Polyglot Input/Output Library                    C
C                                                                     C
C                  projet Estime, INRIA-Rocquencourt                  C
C                                                                     C
C  Copyright 2004-2011 INRIA.                                         C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C $Id: receive_communication.F,v 1.5 2011-03-09 17:57:18 fclement Exp $

C     
C     PIOs receive API.
C
C     Routines for the programmer to receive a communication.
C     
      
#include "pio_constants.h"
      
      subroutine pio_receive_in_communication(ic)
C     Receive a communication from input channel ic.
      
      implicit none
      
C     Input argument.
      integer ic
      
      call scan_communication(ic)
      
      return
      end
      
      
      integer function pio_get_communication_type()
C     Gets the type of the incoming communication.
      
      implicit none
      
C     Local variable.
      character sbr*26
      
C     External function.
      integer fetch_communication_type
      
      sbr = 'pio_get_communication_type'
      
      pio_get_communication_type = fetch_communication_type(sbr, PIO_IN)
      
      return
      end
      
      
      subroutine pio_get_communication_name(len, name)
C     Gets the name of the incoming communication.
C     On input, len is the declared length of the variable name, and on
C     output, len is the length of the communication name.
      
      implicit none
      
C     Input/output argument.
      integer len
      
C     Output argument.
      character name*(*)
      
C     Local variable.
      character sbr*26
      
      sbr = 'pio_get_communication_name'
      
      call fetch_communication_name(sbr, PIO_IN, name, len)
      
      return
      end
      
      
      integer function pio_get_communication_number_of_values()
C     Gets the number of values in the incoming communication.
      
      implicit none
      
C     Local variable.
      character sbr*38
      
C     External function.
      integer fetch_communication_values_number
      
      sbr = 'pio_get_communication_number_of_values'
      
      pio_get_communication_number_of_values =
     +     fetch_communication_values_number(sbr, PIO_IN)
      
      return
      end
      
      
      subroutine pio_get_string(len, sdatum)
C     Gets the current value in the incoming communication, a string.
      
      implicit none
      
C     Input/output argument.
      integer len
      
C     Output argument.
      character sdatum*(*)
      
C     Local variables.
      character sbr*14
      
      sbr = 'pio_get_string'
      
      call seq_fetch_string_value(sbr, PIO_IN, sdatum, len)
      
      return
      end
      
      
      integer function pio_get_int()
C     Gets the current value in the incoming communication, an integer.
      
      implicit none
      
C     Local variables.
      character sbr*11
      
C     External function.
      integer seq_fetch_int_value
      
      sbr = 'pio_get_int'
      
      pio_get_int = seq_fetch_int_value(sbr, PIO_IN)
      
      return
      end
      
      
      FLOAT function pio_get_float()
C     Gets the current value in the incoming communication, a float.
      
      implicit none
      
C     Local variables.
      character sbr*13
      
C     External function.
      FLOAT seq_fetch_float_value
      
      sbr = 'pio_get_float'
      
      pio_get_float = seq_fetch_float_value(sbr, PIO_IN)
      
      return
      end
      
      
      subroutine pio_get_int_vector(size, ivdatum)
C     Gets the current value in the incoming communication, an integer
C     vector ivdatum.
C     On input, size is the declared size of the array variable ivdatum,
C     and on output, size is the size of the current integer vector value.
      
      implicit none
      
C     Input/output argument.
      integer size
      
C     Output argument.
      integer ivdatum(*)
      
C     Local variables.
      character sbr*18
      
      sbr = 'pio_get_int_vector'
      
      call seq_fetch_int_compound_value(sbr, PIO_IN, size, ivdatum)
      
      return
      end
      
      
      subroutine pio_get_float_vector(size, fvdatum)
C     Gets the current value in the incoming communication, a float
C     vector fvdatum.
C     On input, size is the declared size of the array variable fvdatum,
C     and on output, size is the size of the current float vector value.
      
      implicit none
      
C     Input/output argument.
      integer size
      
C     Output argument.
      FLOAT fvdatum(*)
      
C     Local variables.
      character sbr*20
      
      sbr = 'pio_get_float_vector'
      
      call seq_fetch_float_compound_value(sbr, PIO_IN, size, fvdatum)
      
      return
      end
